# -*- coding: utf-8 -*-
 
import numpy as np
import scipy.integrate as integr
import matplotlib.pyplot as plt
import cmath
from math import atan
 
##################################
# un exempe simple pour commencer#
##################################

# définition d'un signal simple  
def e(t):
     return 2+(1)*np.sin( t)+(0.5)*np.sin(3 *t)    
    
    

    
 
######################    
# calcul des coef bn #    
######################
    
bn = np.zeros(4)
for k in [1,3]:
    bn[k] = (1/np.pi) * integr.quad(lambda t,k: e(t)*np.sin(k*t),
 0,2*np.pi,args=(k))[0]    
    

 
######################    
# tracé des spectres #    
######################
plt.figure(1)
plt.subplot(1,2,2)
for k in [1,3]:
    plt.plot([k,k],[0,abs(bn[k])],'b',linewidth=4)
    
plt.plot([0,0],[0,2],'b',linewidth=4) #il s'agit de la valeur moyenne

plt.title('spectre de la fonction e(t)')
    
plt.subplot(1,2,1)
t = np.linspace(0,2*np.pi,500)
plt.plot(t,e(t))
plt.title('graphe de la fonction e(t)')

plt.show()

 

##################################   
# définition de la fonction carré#
##################################    
def carre(t):
    if t<0: return(-1.0)
    else: return(1.0)
    

    
    
    
    
######################    
# calcul des coef bn #    
######################
    
bnCarre = np.zeros(51)
for k in range(1,51):
    bnCarre[k] = 2.0 * integr.quad(lambda t,k: carre(t)*np.sin(2*np.pi*k*t),
 -0.5,0.5,args=(k))[0]

  
######################    
# tracé des spectres #    
######################
plt.figure(2)
for k in range(1,51,2):
    plt.plot([k,k],[0,abs(bnCarre[k])],'b',linewidth=4)
plt.title('spectre de la fonction carre')
plt.show()


##############################################
#tracé des sommes partielles, cas du carré   #
##############################################

n=7 #choix de n
t = np.linspace(-1,1,500)
    
def sommepart(n,t):
    liste_val=[]
    for x in t:
        sn2=0 #il faut bien penser à réinitialiser
        for k in range (1,n+1,2): #on ne prend que les impairs
           sn2=sn2+(bnCarre[k]*np.sin(2*np.pi*k*x)) #on recompose le signal
        liste_val.append(sn2)
    plt.plot(t,liste_val)     
    plt.title('somme partielle de la fonction carré')

plt.figure(3)
sommepart(n,t)
plt.show()  
 

#tracé de diverses sommes partielles, optionnel
plt.figure(4)
for n in [2,5,10,50]:
    t = np.linspace(-1,1,500)
    sn = [ sum(bnCarre[k]*np.sin(2*np.pi*k*x) for k in range(1,n+1)) for x in t]
    plt.plot(t,sn)
plt.title('4 sommes partielles de la fonction carré')

plt.show()
 

###########################
#utilisation de l'existant#
###########################

######################
#un signal cos simple#
######################

fe,T=10,100     #fréquence d'échantillonnage, durée du signal
N=T*fe          #nombre de points d'échantillonnage
t=np.arange(0.,N)/fe    #liste des temps
f0=1                    #fondamental du signal
signal_discret=np.cos(2*np.pi*f0*t)     #signal échantillonné
tcont=np.linspace(0,2,100)              #liste temps resserés pour simuler un signal continu
signal_continu=np.cos(2*np.pi*f0*tcont) #signal continu

fourier=np.fft.fft(signal_discret)/np.size(signal_discret)
freq=np.arange(0.,N)*fe/N   #liste des fréquences

plt.figure(5)
plt.subplot(121)
plt.xlim(0,2)
plt.plot(t,signal_discret,'bo')
plt.plot(tcont,signal_continu,'-k')

plt.xlabel('t(s)')
plt.ylabel('signaux')

 
plt.subplot(1,2,2)
plt.plot(freq,np.abs(fourier),'-k')
plt.title('spectre frequentiel')
plt.xlabel('f(Hz)')
plt.show()

#########################
#un signal plus complexe#
#########################

fe,T=20,100
N=int(T*fe)
t=np.arange(0.,N)/fe
f0=1

fondamental=np.cos(2*np.pi*f0*t)
harmonique2=(2/3)*np.cos(4*np.pi*f0*t)
harmonique4=(8/27)*np.cos(8*np.pi*f0*t)
harmonique6=(32/243)*np.cos(12*np.pi*f0*t)
signal=fondamental+harmonique2+harmonique4+harmonique6



fourier=np.fft.fft(signal)/np.size(signal)
freq=np.arange(0.,N)*fe/N

plt.figure(6)
plt.subplot(121)
plt.xlim(0,2)
plt.plot(t,signal,'bo')
plt.plot(t,signal,'-k')

plt.xlabel('t(s)')
plt.ylabel('signaux')

 
plt.subplot(122)
plt.plot(freq,np.abs(fourier),'-k')
plt.title('spectre frequentiel')
plt.xlabel('f(Hz)')
plt.show()



 
############
# Filtrages#
############

################################
#filtre RC passe bas d'ordre un#
################################

plt.figure(7)
def H(x): return(abs(1.0/(1.0+x*1j)))
t = np.linspace(0,20,200)
plt.subplot(1,2,1)
plt.plot(t,H(t))
plt.xlabel('x')
plt.ylabel('H(x)')

def G(x): return(20.0*np.log10(abs(H(x))))
t = np.linspace(0,100,500)
plt.subplot(1,2,2)
plt.title('diagramme de Bode en gain')
plt.grid()
plt.semilogx(t,G(t))
plt.xlabel('log(x)')
plt.show()


#######################################################################
# effet du filtre passe bas d'ordre 1 sur un signal de pulsation omega#
#######################################################################

plt.figure(8)
h = np.zeros(51) # création des listes contenant les diverses valeur de h et phi
phi = np.zeros(51)
abscisse = np.linspace(-1,1,500) #on trace touhjours pour t entre -1 et 1

 
# réponse du filtre si omega = omega0/10
#on a donc x=1/10

for k in range(51): 
 h[k] = 1/np.sqrt(1.0+(k/10)**2)
 phi[k] = -atan(k/10.0)
sn = [ sum(bnCarre[k]*h[k]*np.sin(2*k*np.pi*t+phi[k]) for k in range(1,51)) 
 for t in abscisse ]

plt.subplot(1,3,1) #sous fenetre gauche
plt.title(r'$\omega=\omega_0/10$')
plt.plot(t,sn)
plt.grid()

# réponse du filtre si omega = omega0
#on a donc x=1

for k in range(51): 
 h[k] = 1/np.sqrt(1.0+k*k)
 phi[k] = -atan(k)

t = np.linspace(-1,1,500)
sn = [ sum(bnCarre[k]*h[k]*np.sin(2*k*np.pi*t+phi[k]) for k in range(1,51)) 
 for t in abscisse ]

plt.subplot(1,3,2) #sous fenetre médiane
plt.title(r'$\omega=\omega_0$')
plt.plot(t,sn)
plt.grid()

# réponse du filtre siomega = 10 omega0(comportement intégrateur)
#on a donc x=10

for k in range(51): 
 h[k] = 1/np.sqrt(1.0+(k*10)**2)
 phi[k] = -atan(k*10.0)
sn = [ sum(bnCarre[k]*h[k]*np.sin(2*k*np.pi*t+phi[k]) for k in range(1,51)) 
 for t in abscisse ]

plt.subplot(1,3,3)  #sous fenetre droite
plt.title(r'$\omega=10\omega_0$')
plt.plot(t,sn)

plt.grid()
plt.show()

##############################################################
# spectre du signal carré filtré par passe bas ordre 1       #
#2 valeurs de w0: une dans la bande passante et une en dehors#
##############################################################

plt.figure(9)

#rappel du spectre initial
plt.subplot(1,3,1)

for k in range(1,51,2):
    plt.plot([k,k],[0,abs(bnCarre[k])],'b',linewidth=4)
    plt.title('spectre de la fonction carre')

# tracé du spectre du signal transmis avec une pulsation dans la bande passante    
plt.subplot(1,3,2)
plt.title(r'$\omega=\omega_0/10$')
for k in range(51): 
 h[k] = 1/np.sqrt(1.0+k*k/100.0)
 phi[k] = -atan(k/10.0)
 
for k in range(1,51,2):
    plt.plot([k,k],[0,bnCarre[k]*h[k]],'b',linewidth=4)
    plt.title('spectre de signal transmis $\omega=\omega_0/10$')

# tracé du spectre du signal transmis avec une pulsation en dehors de la bande passante    
plt.subplot(1,3,3)
plt.title(r'$\omega=10\omega_0$')
for k in range(51): 
 h[k] = 1/np.sqrt(1.0+k*k*100.0)
 phi[k] = -atan(k*10.0)    
for k in range(1,51,2):
   plt.plot([k,k],[0,bnCarre[k]*h[k]],'b',linewidth=4)
   plt.title('spectre de signal transmis $\omega=10\omega_0$')
plt.show()

################################
#effet d'un filtre passe bande #
################################

Q = 100  #valeur du factuer de qualité à modifier

# Fondamental
#plt.subplot(1,2,1)
#plt.title("Fondamental")

#for k in range(1,51):
# (h[k],phi[k]) = cmath.polar(1.0/(1.0+1j*Q*(k/1-1/k)))

h = np.zeros(51)
phi = np.zeros(51)

def filtrepbd(x): 
 return(sum(bnCarre[k]*h[k]*np.sin(2*k*np.pi*x+phi[k]) for k in range(1,51)))
t = np.linspace(-1,1,800)

#plt.plot(t,[filtrepbd(x) for x in t])
#plt.grid()

# Harmonique 3 

plt.figure(10)
for k in range(1,51):
 (h[k],phi[k]) = cmath.polar(1.0/(1.0+1j*Q*(k/3-3/k)))

plt.title("Harmonique 3")
plt.plot(t,[filtrepbd(x) for x in t])
plt.grid()
plt.show()


# On a tjs w0=3w
plt.figure(11)
for k in range(1,51,2):
    plt.plot([k,k],[0,bnCarre[k]*h[k]],'b',linewidth=4)
    plt.title('spectre de signal transmis')
plt.show()
 
######################
# Approche temporelle#
######################

N=1000
f_0,f=2,5
tau=1/(2*np.pi*f_0)
Te=4/(N*f) #période d'échantillonnage

t=np.linspace(0,4/f,N) #echantillonnage de N point sur 4 périodes
 

#calcul de s(t) par resolution de l'ED

s0=0.73     #valeur initiale
sol=[s0]
for n in range (N-1):
    s1=(-s0/tau+5*np.cos(2*np.pi*f*Te*n)/tau+s0/Te)*Te #relation de récurence
    sol.append(s1)
    s0=s1
plt.figure(12)
plt.clf()
plt.plot(t,sol,'g-')
plt.show() 

#signal bruité
fe,T=1000,10 #fréquence d'échantillonnage et durée du signal
N=int(T*fe)
t=np.arange(0.,N)/fe
fondamental=np.sin(2*np.pi*50*t) #signal à 50Hz
bruit=(1/2)*np.sin(2*np.pi*400*t) #♦bruit important à 400 Hz
 
signal=fondamental+bruit

fourier=np.fft.fft(signal)/np.size(signal)
freq=np.arange(0.,N)*fe/N

plt.figure(13)
plt.clf()
plt.subplot(121)
plt.xlim(0,0.1)
plt.plot(t,signal)

plt.xlabel('t(s)')
plt.ylabel('signaux')

 
plt.subplot(122)
plt.plot(freq,np.abs(fourier),'-k')
plt.title('spectre frequentiel')
plt.xlabel('f(Hz)')
plt.show()

w0=2*np.pi*60 #fréquence de coupure à 60Hz
W0Te=w0/fe
Sf=np.zeros(N)
for i in range (1,N):
    Sf[i]=(Sf[i-1]+W0Te*signal[i])/(1+W0Te)
fourierSf=np.fft.fft(Sf)/np.size(Sf)
plt.figure(14)
plt.clf()
plt.subplot(121)
plt.xlim(0,0.1)
plt.plot(t,Sf)

plt.xlabel('t(s)')
plt.ylabel('signaux')

 
plt.subplot(122)
plt.plot(freq,np.abs(fourierSf),'-k')
plt.title('spectre frequentiel')
plt.xlabel('f(Hz)')
plt.show()
    